#ifndef CHOICEDIALOG_H
#define CHOICEDIALOG_H

#include "gmc-common_global.h"

#include <QDialog>

class QAbstractItemModel;

class FilterTreeNodeSortFilterProxyModel;
class ChoiceTreeNodeProxyModel;
class TreeNode;

namespace Ui
{
class AbstractChoiceDialog;
}

class GMC_COMMON_SHARED_EXPORT AbstractChoiceDialog : public QDialog
{
        Q_OBJECT

public:

    AbstractChoiceDialog( QWidget * /*parent*/ = nullptr);
    virtual ~AbstractChoiceDialog();

    QAbstractItemModel *model() const;

    const QSet< int > &acceptColumns() const;
    void setAcceptColumns( const QSet<int> & /*acceptColumns*/ );

protected:

    void afterConstructor();
    void showEvent( QShowEvent * /*event*/ ) override;
    virtual void makeChoice() = 0;

protected slots:

    void selectionChanged();
    void changeDisplayVariant( int /*displayVariant*/ );
    void changeFilter();
    void choice();

protected:

    Ui::AbstractChoiceDialog *_ui;

    QAbstractItemModel *_model;
    FilterTreeNodeSortFilterProxyModel *_proxyModel;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ChoiceDialog : public AbstractChoiceDialog
{

    Q_OBJECT

public:

    ChoiceDialog( QWidget * /*parent*/ = nullptr );
    virtual ~ChoiceDialog();

    void setChoiceModel( ChoiceTreeNodeProxyModel *choiceModel );

protected:

    void makeChoice() override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT SingleChoiceDialog : public AbstractChoiceDialog
{
    Q_OBJECT

public:

    SingleChoiceDialog( QWidget * /*parent*/ = nullptr );
    virtual ~SingleChoiceDialog();

    TreeNode *choosenTreeNode() const;

protected:

    void makeChoice() override;
    void showEvent( QShowEvent * /*event*/ ) override;

protected:

    TreeNode *_choosenTreeNode;

};

//******************************************************************************

#endif // CHOICEDIALOG_H
