#ifndef ELEMENT_H
#define ELEMENT_H

#include "gmc-common_global.h"

#include "treenode.h"

#include <QString>
#include <QIcon>

#include "gmccommon.h"

class QAction;

class Module;
class Element;
class ElementData;
class ElementWidget;

class GMC_COMMON_SHARED_EXPORT ElementRootTreeNode : public TreeNode
{

    Q_OBJECT

public:

    enum ElementColumnsType { NameColumn
                            , _CountColumns };

    ElementRootTreeNode( QObject * /*parent*/ = nullptr );
    virtual ~ElementRootTreeNode();

    void insert( int /*start*/, const QList< TreeNode *> & /*nodes*/ ) override;
    void removeAt( int /*start*/, int /*count*/ = 1 ) override;

    int columnCount() const override;
    QVariant headerData( int /*section*/
                               , Qt::Orientation /*orientation*/
                               , int /*role*/ = Qt::DisplayRole ) const override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ContextOperation
{

public:

    enum ContextType { ElementContext
                     , ElementWidgetContext
                     , _CountColumns };

public:

    ContextOperation( ContextOperation::ContextType /*contextType*/
                    , Module &/*module*/
                    , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ContextOperation();

    Module &module() const;//protected:

    const QString &elementType() const;
    ContextOperation::ContextType contextType() const;

    virtual qulonglong availableOperations() const;
    virtual void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = nullptr
                                 , int /*depth*/ = 0 );

    virtual QString caption() const = 0;

    Element *elementContext();
    ElementWidget *elementWidgetContext();

protected:

    bool isEditable() const;

public:

    static const QString undefinedElementType;
    static const QString allElementType;

protected:

    Module &_module;
    bool ( * _isEditable )( const Module & );
    QString _elementType;

private:

    ContextType _contextType;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT Element
: public TreeNode
, public ContextOperation
{

    Q_OBJECT


public:

    static const QIcon &defaultIcon();

protected:

    static const QString &defaultQuestionForRemove();

public:

    Element( Module & /*module*/
           , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~Element();

    void insert( int /*start*/, const QList< TreeNode *> & /*nodes*/ ) override;
    void removeAt( int /*start*/, int /*count*/ = 1 ) override;

    const QString &groupElementType() const;
    bool isPersistent() const;

    ElementData *elementData() const;
    void setElementData( ElementData * /*elementData*/ );

    virtual ElementWidget *elementWidget( ElementWidgetRole /*role*/ );
    QList< QAction * > elementActions() const;

    int columnCount() const override;
    QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const override;
    bool canFetchMore() const override;
    void fetchMore() override;

    const QString &title() const;
    void setTitle( const QString & /*title*/);

    virtual int displayChildrenVariant() const;
    virtual QList< QString > displayChildrenVariants() const;

    virtual const QString& questionForRemove() const;

protected:

    QString _groupElementType;

    ElementData *_elementData;

    bool _requireLoadChildrens;

    QString _title;

    int _displayChildrenVariant;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT LoginElement : public Element
{

    Q_OBJECT

public:

    LoginElement( Module & /*module*/
                , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~LoginElement();

    qulonglong availableOperations() const override;
    void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = nullptr
                                 , int /*depth*/ = 0 ) override;

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ServiceElement : public Element
{

    Q_OBJECT

protected:

    static const QIcon &startIcon();
    static const QIcon &stopIcon();

public:

    ServiceElement( Module & /*module*/
                  , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ServiceElement();

    QVariant data( int /*column*/, int /*role*/ = Qt::DisplayRole ) const override;
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ContainerElement : public Element
{

    Q_OBJECT

public:

    ContainerElement( Module & /*module*/
                    , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~ContainerElement();

    qulonglong availableOperations() const override;
    void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = nullptr
                                 , int /*depth*/ = 0 ) override;

    void resetRequireLoad();
};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT SupplementLoginMixin
{

public:

    SupplementLoginMixin( Element * /*element*/ );
    ~SupplementLoginMixin();

protected:

    int treeNodeSpecialFlags( int /*specialFlags*/ ) const;

protected:

    Element *_element;
};

//******************************************************************************

//public:

class GMC_COMMON_SHARED_EXPORT OrdinaryElement : public Element

{
    Q_OBJECT


public:

    OrdinaryElement( Module & /*module*/
                   , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~OrdinaryElement();

    qulonglong availableOperations() const override;
    void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = nullptr
                                 , int /*depth*/ = 0 ) override;

    int displayChildrenVariant() const override;
    QList< QString > displayChildrenVariants() const override;

};

//******************************************************************************

//public:

class GMC_COMMON_SHARED_EXPORT SupplementElement : public Element

{
    Q_OBJECT


public:

    SupplementElement( Module & /*module*/
                     , bool ( * /*isEditable*/ )( const Module & ) );
    virtual ~SupplementElement();

    int treeNodeSpecialFlags() const override;

    qulonglong availableOperations() const override;
    void performOperation( OperationType /*operation*/
                                 , ElementData * /*elementData*/ = nullptr
                                 , int /*depth*/ = 0 ) override;

};

//******************************************************************************

#endif // ELEMENT_H
