#ifndef ELEMENTTREEVIEW_H
#define ELEMENTTREEVIEW_H

#include "gmc-common_global.h"

#include <QTableView>
#include <QTreeView>

class Element;

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ElementTableView : public QTableView
{

    Q_OBJECT

signals:

    void currentElementChangedStructure( const QModelIndex & = QModelIndex() );

    void elementContextOperationChangedFocused( Element * );
    void elementContextOperationChangedSelected( Element * );
    void elementContextOperationChangedChanged( Element * );

public:

    explicit ElementTableView( QWidget * /*parent*/ = nullptr);
    virtual ~ElementTableView();

    void setModel(QAbstractItemModel * /*model*/ ) override;

protected:

    void connectModel();
    void disconnectModel();

    void focusInEvent(QFocusEvent * event) override;

protected slots:

    void emitCurrentElementChanged_Selection( const QItemSelection & /*selection*/ );
    void emitCurrentElementChanged_InsertRemove( const QModelIndex & /*index*/ );
    void emitCurrentElementChanged_Changed( const QModelIndex & /*index*/, const QModelIndex & /*index*/ );

};

//******************************************************************************

class GMC_COMMON_SHARED_EXPORT ElementTreeView : public QTreeView
{

    Q_OBJECT

signals:

    void currentElementChangedStructure( const QModelIndex & = QModelIndex() );

    void elementContextOperationChangedFocused( Element * );
    void elementContextOperationChangedSelected( Element * );
    void elementContextOperationChangedChanged( Element * );

public:

    ElementTreeView( QWidget * /*parent*/ = nullptr);
    virtual ~ElementTreeView();

    void setModel(QAbstractItemModel * /*model*/ ) override;

protected:

    void connectModel();
    void disconnectModel();

    void focusInEvent( QFocusEvent * /*event*/ ) override;

protected slots:

    void emitCurrentElementChanged_Selection( const QItemSelection & /*selection*/ );
    void emitCurrentElementChanged_InsertRemove( const QModelIndex & /*index*/ );
    void emitCurrentElementChanged_Changed( const QModelIndex & /*index*/, const QModelIndex & /*index*/ );

};

//******************************************************************************

#endif // ELEMENTTREEVIEW_H
